import os
from os.path import basename, dirname, join, isfile
import subprocess
from typing import List
from glob import glob
import shutil
from osgeo import gdal, osr

from ..util import replace_ext


def calc_step(source_dir: os.PathLike, target_dir: os.PathLike, formulas: List[dict]):
    for i, item in enumerate(formulas):
        # get first filename as a base
        
        filename = next(iter(glob(join(source_dir, item['inputs']['A']['glob']))), None)
        target_filename = join(
            target_dir,
            replace_ext(basename(filename), item.get('output_postfix', '_proc%d' % i) + '.tif', False)
        )
        # fail if a file would be overridden
        # TODO: maybe just log an error
        if isfile(target_filename):
            raise Exception('Calc output filename %s already exists' % target_filename)

        calc_formula(source_dir, item['inputs'], target_filename, item['formula'], item.get('data_type', 'Float32'))

    # take all original files with from the last step
    for filename in glob('%s/*' % source_dir):
        shutil.copy(filename, join(target_dir, basename(filename)))


def calc_formula(source_dir: os.PathLike, inputs: List[dict], target_filename: os.PathLike, formula: str, data_type: str="Float32", nodata_value: float=None):
    cmd = [
        "gdal_calc.py",
        "--calc=%s" % formula,
        "--outfile=%s" % target_filename,
        "--type", data_type,
    ]
    for name in inputs:
         # select first
        filename = next(iter(glob(join(source_dir, inputs[name]['glob']))), None)
        cmd.extend([
            "-%s" % name, filename,
            "--%s_band=%d" % (name, inputs[name].get('band', 1)),
        ])

    if nodata_value is not None:
        cmd.extend("--NoDataValue=%f" % nodata_value)

    subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
